package com.icing.dialer.activities

import android.database.Cursor
import android.os.Bundle
import android.provider.CallLog
import android.telecom.TelecomManager
import android.telecom.PhoneAccountHandle
import android.telecom.PhoneAccount
import android.content.Intent
import android.content.ComponentName
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.FlutterEngine
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import com.icing.dialer.KeystoreHelper
import com.icing.dialer.services.CallService
import com.icing.dialer.services.CallConnectionService

class MainActivity : FlutterActivity() {
    private val KEYSTORE_CHANNEL = "com.example.keystore"
    private val CALLLOG_CHANNEL = "com.example.calllog"
    private val CALL_CHANNEL = "call_service"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        registerPhoneAccount()
        checkAndRequestDefaultDialer()
    }

    private fun registerPhoneAccount() {
        val telecomManager = getSystemService(TELECOM_SERVICE) as TelecomManager
        val phoneAccountHandle = PhoneAccountHandle(
            ComponentName(this, CallConnectionService::class.java),
            "IcingDialerAccount"
        )
        val phoneAccount = PhoneAccount.builder(phoneAccountHandle, "Icing Dialer")
            .setCapabilities(PhoneAccount.CAPABILITY_CALL_PROVIDER)
            .build()
        telecomManager.registerPhoneAccount(phoneAccount)
    }

    override fun configureFlutterEngine(flutterEngine: FlutterEngine) {
        super.configureFlutterEngine(flutterEngine)

        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CALL_CHANNEL).setMethodCallHandler { call, result ->
            when (call.method) {
                "makeGsmCall" -> {
                    val phoneNumber = call.argument<String>("phoneNumber")
                    if (phoneNumber != null) {
                        val success = CallService.makeGsmCall(this, phoneNumber)
                        if (success) {
                            result.success(mapOf("status" to "calling", "phoneNumber" to phoneNumber))
                        } else {
                            result.error("CALL_FAILED", "Failed to initiate call", null)
                        }
                    } else {
                        result.error("INVALID_PHONE_NUMBER", "Phone number is required", null)
                    }
                }
                "hangUpCall" -> {
                    val success = CallService.hangUpCall(this)
                    if (success) {
                        result.success(mapOf("status" to "ended"))
                    } else {
                        result.error("HANGUP_FAILED", "Failed to end call", null)
                    }
                }
                else -> result.notImplemented()
            }
        }

        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, KEYSTORE_CHANNEL)
            .setMethodCallHandler { call, result ->
                KeystoreHelper(call, result).handleMethodCall()
            }

        MethodChannel(flutterEngine.dartExecutor.binaryMessenger, CALLLOG_CHANNEL)
            .setMethodCallHandler { call, result ->
                if (call.method == "getCallLogs") {
                    val callLogs = getCallLogs()
                    result.success(callLogs)
                } else {
                    result.notImplemented()
                }
            }
    }

    private fun checkAndRequestDefaultDialer() {
        val tm = getSystemService(TELECOM_SERVICE) as TelecomManager
        tm.defaultDialerPackage?.let {
            if (it != packageName) {
                val intent = Intent(TelecomManager.ACTION_CHANGE_DEFAULT_DIALER)
                    .putExtra(TelecomManager.EXTRA_CHANGE_DEFAULT_DIALER_PACKAGE_NAME, packageName)
                startActivity(intent)
            }
        }
    }

    private fun getCallLogs(): List<Map<String, Any?>> {
        val logsList = mutableListOf<Map<String, Any?>>()
        val cursor: Cursor? = contentResolver.query(
            CallLog.Calls.CONTENT_URI, null, null, null, CallLog.Calls.DATE + " DESC"
        )
        cursor?.use {
            while (it.moveToNext()) {
                val number = it.getString(it.getColumnIndexOrThrow(CallLog.Calls.NUMBER))
                val type = it.getInt(it.getColumnIndexOrThrow(CallLog.Calls.TYPE))
                val date = it.getLong(it.getColumnIndexOrThrow(CallLog.Calls.DATE))
                val duration = it.getLong(it.getColumnIndexOrThrow(CallLog.Calls.DURATION))

                val map = mutableMapOf<String, Any?>(
                    "number" to number,
                    "type" to type,
                    "date" to date,
                    "duration" to duration
                )
                logsList.add(map)
            }
        }
        return logsList
    }
}